/* Copyright © 2012 Matthew Champion
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in the
 documentation and/or other materials provided with the distribution.
 * Neither the name of mattunderscore.com nor the
 names of its contributors may be used to endorse or promote products
 derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL MATTHEW CHAMPION BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */

package com.mattunderscore.http.headers;

import java.util.ArrayList;
import java.util.Collection;

/**
 * Parse an accept-language field header string to a structured representation.
 * 
 * @author Matt Champion
 * @since 0.0.13
 */
public class LanguageParser implements HeaderParser<QLanguage>
{
    public LanguageParser()
    {
    }

    @Override
    public boolean isCorrectHeader(String header)
    {
        return "ACCEPT-LANGUAGE".equals(header.toUpperCase());
    }

    @Override
    public Collection<QLanguage> parseAll(String header)
            throws UnParsableHeaderException
    {
        if (header == null)
        {
            throw new UnParsableHeaderException("Null Pointer.");
        }
        Collection<QLanguage> qL = new ArrayList<QLanguage>();
        String[] types = header.split(",");
        for (int i = 0; i < types.length; i++)
        {
            QLanguage lang = parse(types[i].trim());
            if (lang != null)
            {
                qL.add(lang);
            }
        }
        return qL;
    }

    @Override
    public QLanguage parse(String types) throws UnParsableHeaderException
    {
        if (types == null)
        {
            throw new UnParsableHeaderException("Null Pointer.");
        }
        if ("".equals(types.trim()))
        {
            return null;
        }
        String[] langParts = types.trim().split("-");
        double qualifier = 1.0;
        if (langParts.length == 1)
        {
            String[] lastParts = langParts[0].trim().split(";");
            if (lastParts.length == 1)
            {
                qualifier = 1.0;
            }
            else if (lastParts.length == 2)
            {
                String[] qualifierParts = lastParts[1].trim().split("=");
                if (qualifierParts.length == 2)
                {
                    if ("q".equals(qualifierParts[0].trim()))
                    {
                        try
                        {
                            qualifier = Double.parseDouble(qualifierParts[1].trim());
                        }
                        catch (NumberFormatException ex)
                        {
                            throw new UnParsableHeaderException("Number format excetpion.");
                        }
                        if (qualifier < 0.0 || qualifier > 1.0)
                        {
                            throw new UnParsableHeaderException(
                                    "Invalid qualifier range.");
                        }
                    }
                }
            }
            else
            {
                throw new UnParsableHeaderException("Incorrect number of parts.");
            }
            return new QLanguage(lastParts[0].trim(), "*", qualifier);
        }
        else if (langParts.length == 2)
        {
            String[] lastParts = langParts[1].trim().split(";");
            if (lastParts.length == 1)
            {
                qualifier = 1.0;
            }
            else if (lastParts.length == 2)
            {
                String[] qualifierParts = lastParts[1].trim().split("=");
                if (qualifierParts.length == 2)
                {
                    if ("q".equals(qualifierParts[0].trim()))
                    {
                        try
                        {
                            qualifier = Double.parseDouble(qualifierParts[1].trim());
                        }
                        catch (NumberFormatException ex)
                        {
                            throw new UnParsableHeaderException("Number format excetpion.");
                        }
                        if (qualifier < 0.0 || qualifier > 1.0)
                        {
                            throw new UnParsableHeaderException(
                                    "Invalid qualifier range.");
                        }
                    }
                }
            }
            else
            {
                throw new UnParsableHeaderException("Incorrect number of parts.");
            }
            return new QLanguage(langParts[0].trim(), lastParts[0].trim(), qualifier);
        }
        else
        {
            throw new UnParsableHeaderException("Unparsable language: "
                    + types.trim() + ": " + langParts.length);
        }
    }
}
