/* Copyright © 2013 Matthew Champion */
/*  */

package com.mattunderscore.http.headers;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

/**
 * A parsed accept encoding header field value with a qualification value.
 * 
 * @author Matt Champion
 * @since 0.1.2
 */
public class QEncoding implements Qualified, HeaderFieldElement
{

    private static final int HASH_SEED = 11;
    private static final int HASH_MULT = 17;

    private final String encoding;
    private final double qualifier;

    /**
     * Parser for HttpServletRequest to QEncoding
     * 
     * @param request The HTTP request to parse
     * @return The List of encodings in the accept-encoding header
     * @throws UnParsableHeaderException
     * @since 0.1.2
     */
    public static List<QEncoding> getRequestEncodings(HttpServletRequest request)
            throws UnParsableHeaderException
    {
        @SuppressWarnings("unchecked")
        Enumeration<String> headerNames = request.getHeaderNames();
        EncodingParser parser = new EncodingParser();
        List<QEncoding> encodings = new ArrayList<QEncoding>();
        while (headerNames.hasMoreElements())
        {
            String headerName = headerNames.nextElement();
            if (parser.isCorrectHeader(headerName))
            {
                @SuppressWarnings("unchecked")
                Enumeration<String> headers = request.getHeaders(headerName);
                while (headers.hasMoreElements())
                {
                    encodings.addAll(parser.parseAll(headers.nextElement()));
                }
            }
        }
        return encodings;
    }

    public QEncoding(final String encoding, final double qualifier)
    {
        this.encoding = encoding;
        this.qualifier = qualifier;
    }

    @Override
    public double getQualifier()
    {
        return qualifier;
    }

    @Override
    public String toString()
    {
        return encoding + ";q=" + qualifier;
    }

    /**
     * Method that returns true if the two content types can be matched against one another. Takes
     * into account wildcard matching.
     * 
     * @param encodingObject
     *            Another encoding
     * @return True if they can be matched
     * @since 0.1.2
     */
    public boolean sameEncoding(QEncoding encodingObject)
    {
        if (encodingObject == null)
        {
            return false;
        }
        else
        {
            return this.encoding.equalsIgnoreCase(encodingObject.encoding)
                    || this.encoding.equals("*") || encodingObject.encoding.equals("*");
        }
    }

    /**
     * Test to see if the encoding is a wildcard.
     * 
     * @return True if the encoding is a wild card.
     * @since 0.1.2
     */
    public boolean isAnyEncoding()
    {
        return "*".equals(encoding);
    }

    @Override
    public int hashCode()
    {
        int hash = HASH_SEED;
        hash = (hash * HASH_MULT) + encoding.hashCode();
        hash = (hash * HASH_MULT) + (int) (qualifier * 10000.0);
        return hash;
    }

    @Override
    public boolean equals(Object object)
    {
        if (object == null)
        {
            return false;
        }
        else if (object instanceof QEncoding)
        {
            QEncoding castObject = (QEncoding) object;
            return this.encoding.equals(castObject.encoding)
                    && this.qualifier == castObject.qualifier;
        }
        else
        {
            return false;
        }
    }
}
